import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

public class Counts2OptionsDialog extends JDialog implements ActionListener, DocumentListener
{
  private int noStates;
  private int ruleTableSize;
  private int percentChange;

  private boolean changed;

  private JLabel noStatesLabel;
  private JTextField noStatesInput;
  private JLabel ruleTableSizeLabel;
  private JTextField ruleTableSizeInput;
  private JLabel percentChangeLabel;
  private JTextField percentChangeInput;
  private JCheckBox clearNewRuleButton;
  private JButton okButton;
  private JButton cancelButton;

  public Counts2OptionsDialog(JFrame owner)
  {
    super(owner, "Options", true);
    noStatesLabel = new JLabel("No States");
    noStatesInput = new JTextField("" + noStates, 3);
    noStatesInput.getDocument().addDocumentListener(this);
    ruleTableSizeLabel = new JLabel("Rule Table Size");
    ruleTableSizeInput = new JTextField("" + ruleTableSize, 3);
    ruleTableSizeInput.getDocument().addDocumentListener(this);
    percentChangeLabel = new JLabel("% Change");
    percentChangeInput = new JTextField("" + percentChange, 3);
    percentChangeInput.getDocument().addDocumentListener(this);
    clearNewRuleButton = new JCheckBox("Clear");
    clearNewRuleButton.setSelected(true);
    okButton = new JButton("Ok");
    okButton.addActionListener(this);
    cancelButton = new JButton("Cancel");
    cancelButton.addActionListener(this);
    JPanel basePanel = new JPanel();
    basePanel.setBorder(BorderFactory.createLineBorder(basePanel.getBackground(), 15));
    JPanel noStatesPanel = new JPanel();
    noStatesPanel.setLayout(new GridLayout(1, 2, 5, 5));
    JPanel noStatesInputPanel = new JPanel();
    noStatesInputPanel.add(noStatesInput);
    noStatesPanel.add(noStatesLabel);
    noStatesPanel.add(noStatesInputPanel);
    noStatesPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), ""));
    JPanel ruleTableSizePanel = new JPanel();
    ruleTableSizePanel.setLayout(new GridLayout(1, 2, 5, 5));
    JPanel ruleTableSizeInputPanel = new JPanel();
    ruleTableSizeInputPanel.add(ruleTableSizeInput);
    ruleTableSizePanel.add(ruleTableSizeLabel);
    ruleTableSizePanel.add(ruleTableSizeInputPanel);
    ruleTableSizePanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), ""));
    JPanel inputPanel = new JPanel();
    inputPanel.setLayout(new GridLayout(2, 1, 5, 5));
    inputPanel.add(noStatesPanel);
    inputPanel.add(ruleTableSizePanel);
    JPanel percentChangePanel = new JPanel();
    percentChangePanel.setLayout(new GridLayout(1, 2, 5, 5));
    JPanel percentChangeInputPanel = new JPanel();
    percentChangeInputPanel.add(percentChangeInput);
    percentChangePanel.add(percentChangeLabel);
    percentChangePanel.add(percentChangeInputPanel);
    JPanel controlButtonPanel = new JPanel();
    controlButtonPanel.setLayout(new GridLayout(1, 1, 5, 5));
    controlButtonPanel.add(clearNewRuleButton);
    JPanel controlPanel = new JPanel();
    controlPanel.setLayout(new BorderLayout(0, 5));
    controlPanel.add(percentChangePanel, BorderLayout.NORTH);
    controlPanel.add(controlButtonPanel, BorderLayout.SOUTH);
    controlPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), "New Rule Options"));
    JPanel buttonPanel = new JPanel();
    buttonPanel.setLayout(new FlowLayout(FlowLayout.LEFT));
    buttonPanel.add(okButton);
    buttonPanel.add(cancelButton);
    basePanel.setLayout(new BorderLayout(0, 5));
    basePanel.add(inputPanel, BorderLayout.NORTH);
    basePanel.add(controlPanel, BorderLayout.CENTER);
    basePanel.add(buttonPanel, BorderLayout.SOUTH);
    add(basePanel);
    pack();
    setResizable(false);
    basePanel.requestFocus();
    changed = false;
  }

  private void setValues()
  {
    try
    {
      int s = new Integer(noStatesInput.getText().trim());
      noStates = s;
      if (noStates < 2)
        noStates = 2;
      if (noStates > 25)
        noStates = 25;
      int t = new Integer(ruleTableSizeInput.getText().trim());
      ruleTableSize = t;
      if (ruleTableSize < 10)
        ruleTableSize = 10;
      if (ruleTableSize > 32)
        ruleTableSize = 32;
      int p = new Integer(percentChangeInput.getText().trim());
      percentChange = p;
      if (percentChange < 1)
        percentChange = 1;
      if (percentChange > 100)
        percentChange = 100;
    }
    catch (Exception e)
    {
    }
  }

  public void setData(Counts2Rule rule)
  {
    noStates = rule.getNoStates();
    noStatesInput.setText("" + noStates);
    ruleTableSize = rule.getRuleTableSize();
    ruleTableSizeInput.setText("" + ruleTableSize);
    percentChange = rule.getPercentChange();
    percentChangeInput.setText("" + percentChange);
    clearNewRuleButton.setSelected(rule.getClearNewRule());
  }

  public void getData(Counts2Rule rule)
  {
    rule.setNoStates(noStates);
    rule.setRuleTableSize(ruleTableSize);
    rule.setPercentChange(percentChange);
    rule.setClearNewRule(clearNewRuleButton.isSelected());
  }

  public boolean hasChanged()
  {
    return changed;
  }

  public void actionPerformed(ActionEvent e)
  {
    Object source = e.getSource();
    if (source == okButton)
    {
      setVisible(false);
      changed = true;
    }
    else if (source == cancelButton)
    {
      setVisible(false);
      changed = false;
    }
  }

  public void insertUpdate(DocumentEvent event)
  {
    setValues();
  }
  
  public void removeUpdate(DocumentEvent event)
  {
    setValues();
  }
  
  public void changedUpdate(DocumentEvent event) {}
}
