import java.awt.*;
import javax.swing.*;
import java.io.*;
import java.util.*;

class CyclicRule extends Rule
{
  private static final int noStateWeights = 16;
  private static final int neighborRows = 5;
  private static final int neighborColumns = 5;

  private int noCycles = 1;
  private int cycle = 0;
  private int noStates;
  private int noCounts;
  private int percentChange = 30;
  private String newRuleOption = "Random";
  private boolean clearNewRule = true;
  private int stateWeights[];
  private int neighborWeights[][][];
  private int ruleTable[][];
  private int useTable[][];

  public CyclicRule(JFrame mainFrame, SquareCell squareCell)
  {
    super(mainFrame, squareCell);
    setDefault();
  }

  private void makeArrays()
  {
    stateWeights = new int[noStateWeights];
    neighborWeights = new int[neighborRows][neighborColumns][noCycles];
    ruleTable = new int[noStates][noCounts];
    useTable = new int[noStates][noCounts];
  }

  private void setWeights()
  {
    for (int r = 0; r < noStateWeights; r++)
      stateWeights[r] = 0;
    stateWeights[1] = 1;
    setNeighborWeights();
  }

  private void setNeighborWeights()
  {
    for (int y = 0; y < noCycles; y++)
    {
      for (int r = 0; r < neighborRows; r++)
        for (int c = 0; c < neighborColumns; c++)
          if ((r > 0) && (r < 4) && (c > 0) && (c < 4))
            neighborWeights[r][c][y] = 1;
          else
            neighborWeights[r][c][y] = 0;
      neighborWeights[2][2][y] = 0;
    }
  }

  private void life()
  {
    noStates = 2;
    noCounts = 9;
    makeArrays();
    setWeights();
    for (int r = 0; r < noStates; r++)
      for (int c = 0; c < noCounts; c++)
          ruleTable[r][c] = 0;
    ruleTable[0][3] = 1;
    ruleTable[1][2] = 1;
    ruleTable[1][3] = 1;
  }

  private void starWars()
  {
    noStates = 4;
    noCounts = 9;
    makeArrays();
    setWeights();
    for (int r = 0; r < noStates; r++)
      for (int c = 0; c < noCounts; c++)
        if (r == 1)
          ruleTable[r][c] = 2;
        else if (r == 2)
          ruleTable[r][c] = 3;
        else
          ruleTable[r][c] = 0;
    ruleTable[0][2] = 1;
    ruleTable[1][3] = 1;
    ruleTable[1][4] = 1;
    ruleTable[1][5] = 1;
  }

  private void wireWorld()
  {
    noStates = 4;
    noCounts = 9;
    makeArrays();
    setWeights();
    for (int r = 0; r < noStates; r++)
      for (int c = 0; c < noCounts; c++)
        if (r == 1)
          ruleTable[r][c] = 2;
        else if ((r == 2) || (r == 3))
          ruleTable[r][c] = 3;
        else
          ruleTable[r][c] = 0;
    ruleTable[3][1] = 1;
    ruleTable[3][2] = 1;
  }

  public void setDefault()
  {
    starWars();
  }

  public void changeRule()
  {
    CyclicDialog cyclicDialog = new CyclicDialog(mainFrame, this, squareCell);
    cyclicDialog.setData(this);
    centerDialog(cyclicDialog);
    cyclicDialog.setVisible(true);
    if (cyclicDialog.hasChanged())
		{
      stateWeights = new int[noStateWeights];
      cyclicDialog.getData(this);
		}
  }

  private void centerDialog(JDialog dialog)
  {
    Point p = mainFrame.getLocation();
    int x = (int)p.getX() + (mainFrame.getWidth() - dialog.getWidth()) / 2;
    int y = (int)p.getY() + (mainFrame.getHeight() - dialog.getHeight()) / 2;
    dialog.setLocation(x, y);
  }

  private void clearUseTable()
  {
    for (int r = 0; r < noStates; r++)
      for (int c = 0; c < noCounts; c++)
        useTable[r][c] = 0;
  }

  public void setNewRule()
  {
    random(ruleTable);
  }

  public void random(int ruleTable[][])
  {
    clearUseTable();
    if (clearNewRule)
    {
      for (int r = 0; r < noStates; r++)
        for (int c = 0; c < noCounts; c++)
          ruleTable[r][c] = 0;
    }
    Random generator = new Random();
    if (newRuleOption.equals("Random"))
    {
      for (int i = 1; i < noStates * noCounts * percentChange / 100; i++)
      {
        int r = generator.nextInt(noStates);
        int c = generator.nextInt(noCounts);
        ruleTable[r][c] = generator.nextInt(noStates);
      }
    }
    else if (newRuleOption.equals("Generations") || newRuleOption.equals("Modified Generations"))
    {
      for (int r = 1; r < (noStates - 1); r++)
        for (int c = 0; c < noCounts; c++)
          ruleTable[r][c] = r + 1;
      if (newRuleOption.equals("Generations"))
        for (int i = 1; i < 2 * noCounts * percentChange / 100; i++)
        {
          int r = generator.nextInt(2);
          int c = generator.nextInt(noCounts);
          ruleTable[r][c] = 1;
        }
      else
        for (int i = 1; i < noStates * noCounts * percentChange / 100; i++)
        {
          int r = generator.nextInt(noStates);
          int c = generator.nextInt(noCounts);
          ruleTable[r][c] = generator.nextInt(noStates);
        }
    }
  }

  public void setOptions()
  {
    int currentNoCycles = noCycles;
    int currentNoStates = noStates;
    int currentNoCounts = noCounts;
    CyclicOptionsDialog cyclicOptionsDialog = new CyclicOptionsDialog(mainFrame);
    cyclicOptionsDialog.setData(this);
    centerDialog(cyclicOptionsDialog);
    cyclicOptionsDialog.setVisible(true);
    if (cyclicOptionsDialog.hasChanged())
    {
      cyclicOptionsDialog.getData(this);
      if (noCycles != currentNoCycles)
      {
        neighborWeights = new int[neighborRows][neighborColumns][noCycles];
        setNeighborWeights();
        squareCell.clear();
      }
      if ((noStates != currentNoStates) || (noCounts != currentNoCounts))
      {
				if (noStates != currentNoStates)
					for (int i = noStates; i < noStateWeights; i++)
					  stateWeights[i] = 0;
        ruleTable = new int[noStates][noCounts];
        useTable = new int[noStates][noCounts];
      }
    }
  }

  public void writeRule(PrintWriter pw)
  {
    pw.println("#Rule = Cyclic");
    pw.println("#Cycles = " + noCycles);
    pw.println("#Cycle = " + cycle);
    pw.println("#States = " + noStates);
    pw.println("#Counts = " + noCounts);
    String s = "#SW ";
    for (int i = 0; i < noStates; i++)
      s = s + stateWeights[i] + ",";
    pw.println(s.substring(0, s.lastIndexOf(",")));
    for (int y = 0; y < noCycles; y++)
      for (int i = 0; i < 5; i++)
      {
        s = "#NW ";
        for (int j = 0; j < 5; j++)
          s = s + neighborWeights[i][j][y] + ",";
        pw.println(s.substring(0, s.lastIndexOf(",")));
      }
    for (int i = 0; i < noStates; i++)
    {
      s = "#RT ";
      for (int j = 0; j < noCounts; j++)
        s = s + ruleTable[i][j] + ",";
      pw.println(s.substring(0, s.lastIndexOf(",")));
    }
  }

  public void readRule(BufferedReader br)
  {
    try
    {
      noCycles = new Integer(br.readLine().substring(10).trim());
      cycle = new Integer(br.readLine().substring(9).trim());
      noStates = new Integer(br.readLine().substring(10).trim());
      noCounts = new Integer(br.readLine().substring(10).trim());
      ruleTable = new int[noStates][noCounts];
      useTable = new int[noStates][noCounts];
			for (int i = 0; i < noStateWeights; i++)
				stateWeights[i] = 0;
      String values[] = br.readLine().substring(3).split(",");
      for (int i = 0; i < noStates; i++)
        stateWeights[i] = new Integer(values[i].trim());
      neighborWeights = new int[neighborRows][neighborColumns][noCycles];
      for (int y = 0; y < noCycles; y++) 
        for (int i = 0; i < 5; i++)
        {
          values = br.readLine().substring(3).split(",");
          for (int j = 0; j < 5; j++)
            neighborWeights[i][j][y] = new Integer(values[j].trim());
        }
      for (int i = 0; i < noStates; i++)
      {
        values = br.readLine().substring(3).split(",");
        for (int j = 0; j < noCounts; j++)
          ruleTable[i][j] = new Integer(values[j].trim());
      }
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }
  }

  public int step(int r, int c)
  {
    int cell = squareCell.getNeighbor(r, c);
    int count =
      stateWeights[squareCell.getNeighbor(r - 2, c - 2)] * neighborWeights[0][0][cycle] +
      stateWeights[squareCell.getNeighbor(r - 2, c - 1)] * neighborWeights[0][1][cycle] +
      stateWeights[squareCell.getNeighbor(r - 2, c    )] * neighborWeights[0][2][cycle] +
      stateWeights[squareCell.getNeighbor(r - 2, c + 1)] * neighborWeights[0][3][cycle] +
      stateWeights[squareCell.getNeighbor(r - 2, c + 2)] * neighborWeights[0][4][cycle] +

      stateWeights[squareCell.getNeighbor(r - 1, c - 2)] * neighborWeights[1][0][cycle] +
      stateWeights[squareCell.getNeighbor(r - 1, c - 1)] * neighborWeights[1][1][cycle] +
      stateWeights[squareCell.getNeighbor(r - 1, c    )] * neighborWeights[1][2][cycle] +
      stateWeights[squareCell.getNeighbor(r - 1, c + 1)] * neighborWeights[1][3][cycle] +
      stateWeights[squareCell.getNeighbor(r - 1, c + 2)] * neighborWeights[1][4][cycle] +

      stateWeights[squareCell.getNeighbor(r    , c - 2)] * neighborWeights[2][0][cycle] +
      stateWeights[squareCell.getNeighbor(r    , c - 1)] * neighborWeights[2][1][cycle] +
      stateWeights[cell                                ] * neighborWeights[2][2][cycle] +
      stateWeights[squareCell.getNeighbor(r    , c + 1)] * neighborWeights[2][3][cycle] +
      stateWeights[squareCell.getNeighbor(r    , c + 2)] * neighborWeights[2][4][cycle] +

      stateWeights[squareCell.getNeighbor(r + 1, c - 2)] * neighborWeights[3][0][cycle] +
      stateWeights[squareCell.getNeighbor(r + 1, c - 1)] * neighborWeights[3][1][cycle] +
      stateWeights[squareCell.getNeighbor(r + 1, c    )] * neighborWeights[3][2][cycle] +
      stateWeights[squareCell.getNeighbor(r + 1, c + 1)] * neighborWeights[3][3][cycle] +
      stateWeights[squareCell.getNeighbor(r + 1, c + 2)] * neighborWeights[3][4][cycle] +

      stateWeights[squareCell.getNeighbor(r + 2, c - 2)] * neighborWeights[4][0][cycle] +
      stateWeights[squareCell.getNeighbor(r + 2, c - 1)] * neighborWeights[4][1][cycle] +
      stateWeights[squareCell.getNeighbor(r + 2, c    )] * neighborWeights[4][2][cycle] +
      stateWeights[squareCell.getNeighbor(r + 2, c + 1)] * neighborWeights[4][3][cycle] +
      stateWeights[squareCell.getNeighbor(r + 2, c + 2)] * neighborWeights[4][4][cycle];
    if ((count >= 0) && (count < noCounts))
    {
      useTable[cell][count] = 1;
      return ruleTable[cell][count];
    }
    if (squareCell.getWeightedGenerations())
    {
      if ((cell > 0) && (cell < (noStates - 1)))
        return cell + 1;
      else
        return 0;
    }
    else
      return 0;
  }

  public void incCycle()
  {
    cycle = (cycle + 1) % noCycles;
  }

  public int getNoCycles()
  {
    return noCycles;
  }

  public int getCycle()
  {
    return cycle;
  }

  public int getNoStates()
  {
    return noStates;
  }

  public int getNoCounts()
  {
    return noCounts;
  }

  public int getPercentChange()
  {
    return percentChange;
  }

  public String getNewRuleOption()
  {
    return newRuleOption;
  }

  public boolean getClearNewRule()
  {
    return clearNewRule;
  }

  public int getStateWeight(int state)
  {
    return stateWeights[state];
  }

  public int getNeighborWeight(int row, int column, int cycle)
  {
    return neighborWeights[row][column][cycle];
  }

  public int getRuleTableValue(int row, int column)
  {
    return ruleTable[row][column];
  }

  public int getUseTableValue(int row, int column)
  {
    return useTable[row][column];
  }

  public void setNoCycles(int noCycles)
  {
    this.noCycles = noCycles;
  }

  public void setCycle(int cycle)
  {
    this.cycle = cycle;
  }

  public void setNoStates(int noStates)
  {
    this.noStates = noStates;
  }

  public void setNoCounts(int noCounts)
  {
    this.noCounts = noCounts;
  }

  public void setPercentChange(int percentChange)
  {
    this.percentChange = percentChange;
  }

  public void setNewRuleOption(String newRuleOption)
  {
    this.newRuleOption = newRuleOption;
  }

  public void setClearNewRule(boolean clearNewRule)
  {
    this.clearNewRule = clearNewRule;
  }

  public void setStateWeight(int state, int value)
  {
    stateWeights[state] = value;
  }

  public void setNeighborWeight(int row, int column, int cycle, int value)
  {
    neighborWeights[row][column][cycle] = value;
  }

  public void setRuleTableValue(int row, int column, int value)
  {
    ruleTable[row][column] = value;
  }

  public void setUseTableValue(int row, int column, int value)
  {
    useTable[row][column] = value;
  }
}
